%ballbounce
% Simulation of a ball bouncing on a horizontal surface. The collision
% is modelled by constant coefficient of restitution C, i.e. the speed of
% separation from the floor / speed of approach is a fixed ratio. e.g. 1
% for a purely elastic collision, 0.5 for a more typical bounce of a tennis
% ball.
%
% LAST UPDATED by Andy French May 2016

function ballbounce

%Define inputs

%Initial height of ball /m (dropped with zero velocity)
h = 1.2;

%Strength of gravity /ms^-2
g = 9.81;

%Coefficient of restitution
C = 0.7893;

%Number of plot points between bounces
N = 1000;

%Max number of bounces
num_bounces = 10;

%Fontsize for graphs
fsize = 18;

%Frame rate for animation 
FPS = 100;

%

%Compute initial impact velocity u /ms^-1
u = sqrt( 2*g*h );

%Complete initial fall time /s
tf = sqrt(2*h/g);

%Compute total distance travelled /m and total bounce time /s
D = 2*h*( 1/(1 - C^2) - 1/2 );
T = 2*tf*( 1/(1-C) - 1/2 );

%Determine displacement and velocity vectors for the initial drop
t = linspace(0,tf,N);
x = h - 0.5*g*t.^2;
v = -g*t;
bounce_time = tf;

%Step through bounce times and populate displacement and velocity vectors
for n=1:num_bounces
    
    %Define time /s since bounce
    tt = linspace( 0, 2*tf*(C^n), N);
    
    %Update array of bounce tims
    bounce_time = [ bounce_time, bounce_time(end) + 2*tf*(C^n) ];
    
    %Update time array
    t = [t, t(end) + tt];
    
    %Determine displacement and velocity
    x = [x, u*(C^n)*tt - 0.5*g*tt.^2];
    v = [v, u*(C^n) - g*tt];
end

%Plot displacement vs time graph
fig = figure('name','ball bounce','color',[1 1 1]);
plot(t,x);
hold on;
plot( bounce_time,zeros(size(bounce_time)),'r*');
xlabel('time t /s','fontsize',fsize)
ylabel('displacement x /m','fontsize',fsize)
title(['Ball bounce. C = ',num2str(C),...
    ', T = ',num2str(T),'s, D = ',num2str(D),'m'],'fontsize',fsize);
set(gca,'fontsize',fsize);
grid on;
print(gcf,['bounce x C=',strrep(num2str(C),'.','p'),'.png'],'-dpng','-r300');
clf;

%Plot velocity vs time graph
plot(t,v);
xlabel('time t /s','fontsize',fsize)
ylabel('velocity v /ms^{-1}','fontsize',fsize)
title(['Ball bounce. C = ',num2str(C),...
    ', T = ',num2str(T),'s, D = ',num2str(D),'m'],'fontsize',fsize);
set(gca,'fontsize',fsize);
grid on;
print(gcf,['bounce v C=',strrep(num2str(C),'.','p'),'.png'],'-dpng','-r300');
clf;

%

%Loop animation of falling ball
plot(0,h,'k*');
hold on;
stop = 0;
theta = linspace(0,2*pi,30);
xc = (h/20)*cos(theta);
yc = (h/20)*sin(theta);
p = patch(xc,yc,'g');
axis equal
axis  manual
xlim([-3*(h/20),3*(h/20)]);
ylim([0,h+h/20]);
n=1;
set(gcf,'renderer','opengl')
xlabel('x','fontsize',fsize)
xlabel('y','fontsize',fsize)
tit = title({['C = ',num2str(C)],['t = 0s']},'fontsize',fsize);
set(gca,'fontsize',fsize)

%Interpolate displacement at frame rate
ti = 0 : 1/FPS : T;
[t,i] = unique(t);
x = x(i);
xi = interp1( t,x,ti);
global d
d.run = 1;
set(fig,'keypressfcn',@keypressfunc);
while ( d.run == 1 ) && ( ishandle(fig) == 1 )
    set(p, 'ydata', yc + xi(n) + h/20);
    set(tit,'string',{['C = ',num2str(C)],['t = ',num2str(ti(n),2),'s']})
    n = n + 1;
    if n==length(xi)
        n = 1;
    end
    drawnow
    pause(1/FPS);
end

%%

% Key press handler
function keypressfunc( fig,evnt )
global d
if strcmp(get(gcf,'currentkey'),'q')==1
    d.run = 0;
    close(fig);
end

%End of code