%rocket
% A model of a rocket. 1D only, i.e. effects of windage is ignored.
% LAST UPDATED by Andy French January 2020
function rocket

%Fuel mass /kg
m0 = 0.3;

%Empty rocket mass /kg
M = 0.3;

%Strength of gravity /ms^-2
g = 9.81;

%Water ejection rate (assumed constant) /kg/s
mu = 0.11;

%Fuel ejection speed (assumed constant) /ms^-1
C = 220;

%Radius of rocket /m
r = 5e-2;

%Drag coefficient 
cD = 0.5;

%Density of air /kgm^-3
rho = 1.225;

%Timestep /s
dt = 1e-4;

%Fontsize for graphs
fsize = 18;

%

%Initialize variables
x = 0; v = 0; t = 0; n=1; a = rocket_acc( t,v,M,m0,mu,C,g,rho,cD,r );

%Compute projectile t,x,v,a using Verlet method
while x(end)>=0
    t(n+1) = t(n) + dt;
    x(n+1) = x(n) +v(n)*dt + 0.5*a(n)*( dt^2 );
    V = v(n) + a(n)*dt;
    av = rocket_acc(  t(n+1),V,M,m0,mu,C,g,rho,cD,r );
    v(n+1) = v(n) + 0.5*( a(n) + av )*dt;
    a(n+1) = rocket_acc(  t(n+1),v(n+1),M,m0,mu,C,g,rho,cD,r );
    n = n+1;
end

%Determine thrust phase
thrust = find( t < m0/mu );

%Determine apogee and time of flight
xmax = max(x);
T = t(end);

%Plot dynamics

%x vs t
plot(t,x,'linewidth',2); set(gca,'fontsize',fsize); grid on;
hold on; plot( t(thrust),x(thrust),'r','linewidth',2);
xlabel('t/s','fontsize',fsize); ylabel('x/m','fontsize',fsize);
title({['Rocket model: M=',num2str(M),'kg, m_0=',num2str(m0),'kg, \mu=',num2str(mu),'kg/s, ',...
    'C=',num2str(C),'m/s.'],...
    ['Time of flight=',num2str(t(end),3),...
    's, thrust time=',num2str(m0/mu,3),'s, apogee=',num2str(xmax,3),'m.']},'fontsize',fsize);
ylim([0,xmax]); xlim([0,T]);
print(gcf,'rocket x vs t.png','-dpng','-r300'); cla;

%v vs t
plot(t,v,'linewidth',2); set(gca,'fontsize',fsize); grid on;
hold on; plot( t(thrust),v(thrust),'r','linewidth',2);
xlabel('t/s','fontsize',fsize); ylabel('v/ms^{-1}','fontsize',fsize);
title({['Rocket model: M=',num2str(M),'kg, m_0=',num2str(m0),'kg, \mu=',num2str(mu),'kg/s, ',...
    'C=',num2str(C),'m/s.'],...
    ['Time of flight=',num2str(t(end),3),...
    's, thrust time=',num2str(m0/mu,3),'s, apogee=',num2str(max(x),3),'m.']},'fontsize',fsize);
xlim([0,T]); axis tight
print(gcf,'rocket v vs t.png','-dpng','-r300'); cla;

%a vs t
plot(t,a,'linewidth',2); set(gca,'fontsize',fsize); grid on;
hold on; plot( t(thrust),a(thrust),'r','linewidth',2);
xlabel('t/s','fontsize',fsize); ylabel('a/ms^{-2}','fontsize',fsize);
title({['Rocket model: M=',num2str(M),'kg, m_0=',num2str(m0),'kg, \mu=',num2str(mu),'kg/s, ',...
    'C=',num2str(C),'m/s.'],...
    ['Time of flight=',num2str(t(end),3),...
    's, thrust time=',num2str(m0/mu,3),'s, apogee=',num2str(max(x),3),'m.']},'fontsize',fsize);
xlim([0,T]); axis tight
print(gcf,'rocket a vs t.png','-dpng','-r300'); close(gcf);
    
%%

%rocket acceleration (via Newton II)
function a = rocket_acc( t,v,M,m0,mu,C,g,rho,cD,r )
if t<m0/mu
    %Thrust phase
    a = -g + ( mu*C - 0.5*rho*cD*pi*(r^2)*v*abs(v) )/( M + m0 - mu*t );
else
    %Projectile phase
    a = -g  - 0.5*rho*cD*pi*(r^2)*v*abs(v)/M;
end

%End of code